CON
           
' *******************************************************************************************************
' *                                                                                                     *
' *     AiChip I2C Slave Interface                                       AiChip_I2cSlave_Slave_003      *
' *                                                                                                     *
' *******************************************************************************************************
' *                                                                                                     *
' *     Object Name    : AiChip I2C Slave Interface ( Emulates 256 byte I2C Ram, 16-bit addressing )    *
' *     Version        : 003                                                                            *
' *     Updated        : 2008-04-12                                                                     *
' *                                                                                                     *
' *     Author         : AiChip Industries                                                              *
' *                                                                                                     *
' *     Copyright      : 2008, AiChip Industries                                                        *
' *                                                                                                     *
' *     Target Chip    : Parallax Propeller Chip P8X32A ( "Propeller Mk I" )                            *
' *     Compiler Used  : Parallax Propeller Tool 1.1                                                    *
' *                                                                                                     *
' *     License        : MIT License. Please see end of this file for Terms of Use.                     *
' *                                                                                                     *
' *******************************************************************************************************
' *                                                                                                     *
' *     AiChip is a trademark of AiChip Industries.                                                     *
' *                                                                                                     *
' *     If distributing a modified version of this software AiChip Industries requests the "AiChip"     *
' *     prefix be removed from the filename and internal text descriptions to avoid any confusion that  *
' *     may be caused by an impression that the modified software originates from AiChip Industries.    *
' *                                                                                                     *
' *******************************************************************************************************

' *******************************************************************************************************
' *                                                                                                     *
' *     HISTORY                                                                                         *
' *                                                                                                     *
' *     003     2008-04-12      Changes to PASM version of I2C Slave - Now working                      *
' *                             Ack and Nak polarity corrected                                          *
' *                             PASM parameters passed on Spin stack                                    *
' *                             Spin version removed                                                    *
' *                                                                                                     *
' *     002     2008-04-12      PASM version of I2C Slave included - Not working yet                    *
' *                                                                                                     *
' *     001     2008-04-11      First Public Release                                                    *
' *                                                                                                     *
' *******************************************************************************************************

PUB Version

  result := String( "AiChip_I2cSlave_Slave_003" )

VAR

  byte CogNumber

PUB Start( sclPin, sdaPin, id, a2, a1, a0 )

  Stop

  id := ( id & $FE ) | a2 << 3 | a1 << 2 | a0 << 1

  cogNumber := CogNew( @Pasm_I2cSlave, @sclPin ) + 1

  repeat while sclPin => 0
  
PUB Stop

  if cogNumber
    CogStop( cogNumber~ - 1 )

CON

' *******************************************************************************************************
' *                                                                                                     *
' *     PASM Handler                                                                                    *
' *                                                                                                     *
' *******************************************************************************************************
                        
DAT                     org     $000

'                       .-------------------------------------------------------------------------------.
'                       |       initialise the parameters                                               |
'                       `-------------------------------------------------------------------------------'

Pasm_I2cSlave           mov     pasm_idx,PAR

                        call    #Pasm_GetParam                                  
                        mov     pasm_sclMask,#1
                        shl     pasm_sclMask,pasm_byte                        

                        call    #Pasm_GetParam                                  
                        mov     pasm_sdaMask,#1
                        shl     pasm_sdaMask,pasm_byte                        

                        call    #Pasm_GetParam                                  
                        and     pasm_byte,#$FE
                        mov     pasm_deviceWr,pasm_byte
                        or      pasm_byte,#1
                        mov     pasm_deviceRd,pasm_byte

'                       .-------------------------------------------------------------------------------.
'                       |       Tell caller the cog is running                                          |
'                       `-------------------------------------------------------------------------------'

                        neg     pasm_byte,#1
                        wrlong  pasm_byte,PAR
                        
'                       .-------------------------------------------------------------------------------.
'                       |       Wait for a start bit                                                    |
'                       `-------------------------------------------------------------------------------'
                        
Pasm_Idle               call    #Pasm_GetBit
                        jmp     #Pasm_Idle

'                       .-------------------------------------------------------------------------------.
'                       |       Determine if command is for us or not                                   |
'                       `-------------------------------------------------------------------------------'

Pasm_GotStartBit        call    #Pasm_GetByte
                        cmp     pasm_byte,pasm_deviceRd WZ
                  IF_Z  jmp     #Pasm_Read
                        cmp     pasm_byte,pasm_deviceWr WZ
                  IF_Z  jmp     #Pasm_Write
                        jmp     #Pasm_Idle

'                       .-------------------------------------------------------------------------------.
'                       |       Handle a Read command                                                   |
'                       `-------------------------------------------------------------------------------'

Pasm_Read               call    #Pasm_Ack

:Loop                   call    #Pasm_LoadByte
                        call    #Pasm_SendByte
                        
                        call    #Pasm_GetBit
                        tjz     pasm_bit,#:Loop
                        jmp     #Pasm_Idle

'                       .-------------------------------------------------------------------------------.
'                       |       Handle a Write command                                                  |
'                       `-------------------------------------------------------------------------------'

Pasm_Write              call    #Pasm_Ack

                        call    #Pasm_GetByte                                   ' Address High byte
                        mov     pasm_adr,pasm_byte
                        call    #Pasm_Ack
                        
                        call    #Pasm_GetByte                                   ' Address Low byte
                        shl     pasm_adr,#8
                        or      pasm_adr,pasm_byte
                        call    #Pasm_Ack
                        
:Loop                   call    #Pasm_GetByte                                   ' Data byte
                        Call    #Pasm_SaveByte
                        call    #Pasm_Ack                                              
                        jmp     #:Loop

'                       .-------------------------------------------------------------------------------.
'                       |       Handle saving and loading from emulated 256 byte I2C Ram                |
'                       `-------------------------------------------------------------------------------'

Pasm_SaveByte           mov     pasm_idx,pasm_adr
                        and     pasm_idx,#$FF
                        add     pasm_idx,#pasm_ram     
                        movd    :Dst,pasm_idx
                        add     pasm_adr,#1
:Dst                    mov     0-0,pasm_byte
Pasm_SaveByte_Ret       ret

Pasm_LoadByte           mov     pasm_idx,pasm_adr
                        and     pasm_idx,#$FF
                        add     pasm_idx,#pasm_ram     
                        movs    :Src,pasm_idx
                        add     pasm_adr,#1
:Src                    mov     pasm_byte,0-0
Pasm_LoadByte_Ret       ret

'                       .-------------------------------------------------------------------------------.
'                       |       Send Ack or Nak                                                         |
'                       `-------------------------------------------------------------------------------'
                                                
Pasm_Ack                mov     pasm_byte,#0
                        call    #Pasm_SendBit
Pasm_Ack_Ret            ret

Pasm_Nak                mov     pasm_byte,#$80
                        call    #Pasm_SendBit
Pasm_Nak_Ret            ret

'                       .-------------------------------------------------------------------------------.
'                       |       Send a byte                                                             |
'                       `-------------------------------------------------------------------------------'

Pasm_SendByte           mov     pasm_bitCount,#8

:Loop                   call    #Pasm_SendBit
                        shl     pasm_byte,#1
                        djnz    pasm_bitCount,#:Loop

Pasm_SendByte_Ret       ret

'                       .-------------------------------------------------------------------------------.
'                       |       Send a bit                                                              |
'                       `-------------------------------------------------------------------------------'

Pasm_SendBit            test    pasm_sclMask,INA WC
                  IF_C  jmp     #$-1

                        test    pasm_byte,#$80 WC
                  IF_NC mov     DIRA,pasm_sdaMask
                  
                        test    pasm_sclMask,INA WC
                  IF_NC jmp     #$-1
                  
                        test    pasm_sclMask,INA WC
                  IF_C  jmp     #$-1
                  
                        mov     DIRA,#0

Pasm_SendBit_Ret        ret

'                       .-------------------------------------------------------------------------------.
'                       |       Read a byte                                                             |
'                       `-------------------------------------------------------------------------------'

Pasm_GetByte            mov     pasm_bitCount,#8
                        mov     pasm_byte,#0
                        
:Loop                   call    #Pasm_GetBit
                        shl     pasm_byte,#1
                        or      pasm_byte,pasm_bit
                        
                        djnz    pasm_bitCount,#:Loop

Pasm_GetByte_Ret        ret

'                       .-------------------------------------------------------------------------------.
'                       |       Determine bit or event from the incoming bit stream                     |
'                       `-------------------------------------------------------------------------------'

Pasm_GetBit             call    #Pasm_GetBitStream                              ' Match the bit stream

                        cmp     pasm_bitStream,#%11_10_00 WZ                    ' Start bit
                  IF_Z  jmp     #Pasm_GotStartBit

                        cmp     pasm_bitStream,#%00_10_11 WZ                    ' Stop bit
                  IF_NZ jmp     #:Check0

:WaitForStart           call    #Pasm_GetBitStream                              '   Wait until start bit
                        cmp     pasm_bitStream,#%11_10_00 WZ
                  IF_Z  jmp     #Pasm_GotStartBit
                        jmp     #:WaitForStart

:Check0                 cmp     pasm_bitStream,#%00_10_00 WZ                    ' Bit 0
                  IF_NZ jmp     #:Check1

                        mov     pasm_bit,#0
                        jmp     #Pasm_GetBit_Ret
                        
:Check1                 cmp     pasm_bitStream,#%01_11_01 WZ                    ' Bit 1
                  IF_NZ jmp     #Pasm_GetBit

                        mov     pasm_bit,#1
                        
Pasm_GetBit_Ret         ret

'                       .-------------------------------------------------------------------------------.
'                       |       Read a bit pair into the bit stream                                     |
'                       `-------------------------------------------------------------------------------'

Pasm_GetBitStream       mov     pasm_inputs,INA                                 ' Read SCLK/SDA
                        test    pasm_inputs,pasm_sclMask WC
                        muxc    pasm_thisBits,#2
                        test    pasm_inputs,pasm_sdaMask WC
                        muxc    pasm_thisBits,#1
                       
                        cmp     pasm_thisBits,pasm_bitInputs WZ                 ' Same as last time ?
                  IF_Z  jmp     #Pasm_GetBitStream                              ' Yes - Wait until changed

                        mov     pasm_bitInputs,pasm_thisBits                    ' Remember this state

                        shl     pasm_bitStream,#2                               ' Update the bit-stream                       
                        and     pasm_bitStream,#%11_11_00                       ' Keep three samples
                        or      pasm_bitStream,pasm_bitInputs
                        
Pasm_GetBitStream_Ret   ret

'                       .-------------------------------------------------------------------------------.
'                       |       Get a parameter                                                         |
'                       `-------------------------------------------------------------------------------'

Pasm_GetParam           rdlong  pasm_byte,pasm_idx
                        add     pasm_idx,#4
Pasm_GetParam_Ret       ret

'                       .-------------------------------------------------------------------------------.
'                       |       Pre-Initialised Variables                                               |
'                       `-------------------------------------------------------------------------------'

pasm_bitInputs          long    %11
pasm_thisBits           long    %11
pasm_bitStream          long    %11_11_11

'                       .-------------------------------------------------------------------------------.
'                       |       Variables                                                               |
'                       `-------------------------------------------------------------------------------'

pasm_sclMask            res     1
pasm_sdaMask            res     1
pasm_deviceRd           res     1
pasm_deviceWr           res     1

pasm_inputs             res     1

pasm_bitCount           res     1
pasm_bit                res     1
pasm_byte               res     1

pasm_adr                res     1
pasm_idx                res     1

pasm_ram                res     256

                        fit     $1F0

CON

' *******************************************************************************************************
' *                                                                                                     *
' *     Terms of Use : MIT License                                                                      *
' *                                                                                                     *
' *******************************************************************************************************
'
' Permission is hereby granted, free of charge, to any person obtaining a copy of this software and
' associated documentation files (the "Software"), to deal in the Software without restriction, including
' without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
' copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to
' the following conditions:                                                                   
'
' The above copyright notice and this permission notice shall be included in all copies or substantial
' portions of the Software.                                                                                             
'                                                                                                                  
' THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT
' LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
' NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
' WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
' SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

CON

' *******************************************************************************************************
' *                                                                                                     *
' *     End of AiChip I2C Slave Interface                                                               *
' *                                                                                                     *
' *******************************************************************************************************
  